import { defineComponent, ref, watch, inject, nextTick, resolveComponent, openBlock, createElementBlock, withModifiers, unref, withDirectives, createElementVNode, normalizeClass, normalizeStyle, renderSlot, Fragment, createTextVNode, createCommentVNode, withKeys, vModelText, toDisplayString, vShow, renderList, createBlock, withCtx, provide, createVNode, onMounted, onBeforeUnmount, computed, Teleport, Transition } from 'vue';

var Position = /* @__PURE__ */ ((Position2) => {
  Position2["ABOVE"] = "above";
  Position2["IN"] = "in";
  Position2["BELOW"] = "below";
  return Position2;
})(Position || {});

const _hoisted_1$1 = ["draggable"];
const _hoisted_2 = { class: "title" };
const _sfc_main$3 = /* @__PURE__ */ defineComponent({
  __name: "FileTreeNode",
  props: {
    nodeData: {
      type: Object,
      required: true
    },
    level: {
      type: Number,
      required: true
    },
    draggable: {
      type: Boolean,
      default: false
    },
    levelMargin: {
      type: Number,
      default: 28
    }
  },
  emits: [
    "nodeDrop",
    "nodeSelect",
    "fileCreate",
    "folderCreate",
    "nodeRename",
    "nodeContextmenu",
    "nodeExpand",
    "nodeCollapse",
    "nodeUpdate"
  ],
  setup(__props, { emit: __emit }) {
    const props = __props;
    const emits = __emit;
    let nodeDataInner = ref(props.nodeData);
    watch(() => props.nodeData, () => {
      nodeDataInner.value = props.nodeData;
      newName.value = props.nodeData.title;
    });
    watch(nodeDataInner, () => {
      emits("nodeUpdate", nodeDataInner);
    }, { deep: true });
    const ddo = inject("ddo");
    const editingInputRef = ref();
    const addingFolderInputRef = ref();
    const addingFileInputRef = ref();
    const dragOverClass = ref("");
    const newFileName = ref("");
    const newFolderName = ref("");
    const newName = ref(nodeDataInner.value.title);
    watch(
      () => nodeDataInner.value.editing,
      async (newValue) => {
        if (newValue) {
          newName.value = nodeDataInner.value?.title;
          await nextTick();
          editingInputRef.value.focus();
        }
      }
    );
    watch(
      () => nodeDataInner.value.addingFolder,
      async (newValue) => {
        if (newValue) {
          newFolderName.value = "";
          await nextTick();
          addingFolderInputRef.value.focus();
        }
      }
    );
    watch(
      () => nodeDataInner.value.addingFile,
      async (newValue) => {
        if (newValue) {
          newFileName.value = "";
          await nextTick();
          addingFileInputRef.value.focus();
        }
      }
    );
    const onNodeContextmenu = async (e, nodeData) => {
      emits("nodeContextmenu", e, nodeData);
    };
    function onNodeToggle() {
      nodeDataInner.value.expanded = !nodeDataInner.value.expanded;
      if (nodeDataInner.value.expanded) {
        emits("nodeExpand", nodeDataInner);
      } else {
        emits("nodeCollapse", nodeDataInner);
      }
    }
    function onNodeRename(node, name) {
      if (node.editing) {
        if (name.length > 0) {
          emits("nodeRename", node, name);
        } else {
          onEditingEsc();
        }
      }
    }
    function onEditingEsc() {
      nodeDataInner.value.editing = false;
    }
    function onNodeDrop() {
      emits("nodeDrop");
    }
    function onNodeSelect(e, nodeData) {
      const isCtrlOrCmdSelect = e.ctrlKey || e.metaKey;
      const isShiftSelect = e.shiftKey;
      if (nodeData.type === "folder" && !isCtrlOrCmdSelect && !isShiftSelect && nodeData.path === props.nodeData.path) {
        onNodeToggle();
        if (!props.draggable)
          return;
      }
      emits("nodeSelect", e, nodeData);
    }
    function onFileCreate(nodeData, newFileName2) {
      if (nodeData.addingFile) {
        nodeData.expanded = true;
        if (newFileName2.length > 0) {
          emits("fileCreate", nodeData, newFileName2);
        } else {
          onFileEsc();
        }
      }
    }
    function onFileEsc() {
      nodeDataInner.value.expanded = true;
      nodeDataInner.value.addingFile = false;
    }
    function onFolderCreate(nodeData, newFileName2) {
      if (nodeData.addingFolder) {
        nodeData.expanded = true;
        if (newFileName2.length > 0) {
          emits("folderCreate", nodeData, newFileName2);
        } else {
          onFileEsc();
        }
      }
    }
    function onFolderEsc() {
      nodeDataInner.value.expanded = true;
      nodeDataInner.value.addingFolder = false;
    }
    function onNodeExpand(nodeData) {
      emits("nodeExpand", nodeData);
    }
    function onNodeCollapse(nodeData) {
      emits("nodeCollapse", nodeData);
    }
    function onNodeUpdate(nodeData) {
      const index = nodeDataInner.value.children?.findIndex((item) => item.path === nodeData.path) ?? -1;
      if (index >= 0) {
        nodeDataInner.value.children?.splice(index, 1, nodeData);
      }
    }
    function onDragStart() {
      ddo.drag = nodeDataInner;
    }
    function onDragOver(e) {
      if (e.dataTransfer) {
        e.dataTransfer.dropEffect = "move";
      }
      if (nodeDataInner.value?.path === ddo.drag.path) {
        return;
      }
      const position = calculateDropPosition(e, nodeDataInner.value?.type);
      ddo.position = position;
      if (position === Position.ABOVE) {
        dragOverClass.value = "tree-drag-over-top";
      } else if (position === Position.BELOW) {
        if (nodeDataInner.value.type === "file" || !nodeDataInner.value.expanded) {
          dragOverClass.value = "tree-drag-over-bottom";
        }
      } else {
        dragOverClass.value = "tree-drag-over";
      }
    }
    function calculateDropPosition(e, type) {
      const targetElement = e.target;
      const offsetTop = getOffset(targetElement).top;
      const { offsetHeight } = targetElement;
      const { pageY } = e;
      const gapHeight = type === "folder" ? 0.2 * offsetHeight : 0.5 * offsetHeight;
      if (pageY >= offsetTop + offsetHeight - gapHeight) {
        return Position.BELOW;
      }
      if (pageY < offsetTop + gapHeight) {
        return Position.ABOVE;
      }
      return Position.IN;
    }
    function getOffset(ele) {
      if (!ele.getClientRects().length) {
        return { top: 0, left: 0 };
      }
      const rect = ele.getBoundingClientRect();
      if (rect.width || rect.height) {
        const doc = ele.ownerDocument;
        const win = doc.defaultView;
        const docElem = doc.documentElement;
        if (win) {
          return {
            // 元素距离视窗顶部距离，滚动高度，元素边框厚度
            top: rect.top + win.scrollY - docElem.clientTop,
            left: rect.left + win.scrollX - docElem.clientLeft
          };
        }
      }
      return rect;
    }
    function onDragLeave() {
      dragOverClass.value = "";
    }
    function onDrop() {
      ddo.drop = nodeDataInner;
      emits("nodeDrop");
      dragOverClass.value = "";
      return;
    }
    function onDragEnd() {
      dragOverClass.value = "";
    }
    return (_ctx, _cache) => {
      const _component_FileTreeNode = resolveComponent("FileTreeNode", true);
      return openBlock(), createElementBlock("li", {
        draggable: `${__props.draggable}`,
        class: "file-tree-node",
        onDragstart: withModifiers(onDragStart, ["stop"]),
        onDragend: withModifiers(onDragEnd, ["stop"]),
        onContextmenu: _cache[12] || (_cache[12] = withModifiers(($event) => onNodeContextmenu($event, unref(nodeDataInner)), ["stop", "prevent"]))
      }, [
        withDirectives(createElementVNode("div", {
          class: normalizeClass([
            "node-block",
            {
              selected: unref(nodeDataInner).selected,
              focused: unref(nodeDataInner).focused,
              leaf: !unref(nodeDataInner).children?.length,
              folder: unref(nodeDataInner).type === "folder"
            }
          ]),
          onDragenter: _cache[4] || (_cache[4] = withModifiers(() => {
          }, ["stop"])),
          onDragover: withModifiers(onDragOver, ["stop", "prevent"]),
          onDragleave: withModifiers(onDragLeave, ["stop"]),
          onDrop: withModifiers(onDrop, ["stop"]),
          onClick: _cache[5] || (_cache[5] = withModifiers(($event) => onNodeSelect($event, unref(nodeDataInner)), ["stop"]))
        }, [
          createElementVNode("div", {
            style: normalizeStyle({ "margin-left": (props.level < 3 ? 0 : (props.level - 2) * props.levelMargin) + "px" }),
            class: normalizeClass([dragOverClass.value])
          }, [
            unref(nodeDataInner).type === "folder" ? (openBlock(), createElementBlock("span", {
              key: 0,
              class: "icon",
              onDragover: _cache[0] || (_cache[0] = withModifiers(() => {
              }, ["prevent"]))
            }, [
              renderSlot(_ctx.$slots, "toggler", { nodeData: unref(nodeDataInner) }, () => [
                unref(nodeDataInner).expanded ? (openBlock(), createElementBlock(Fragment, { key: 0 }, [
                  createTextVNode("-")
                ], 64)) : (openBlock(), createElementBlock(Fragment, { key: 1 }, [
                  createTextVNode("+")
                ], 64))
              ])
            ], 32)) : createCommentVNode("", true),
            createElementVNode("span", null, [
              renderSlot(_ctx.$slots, "icon", { nodeData: unref(nodeDataInner) }, () => [
                unref(nodeDataInner).type === "folder" ? (openBlock(), createElementBlock(Fragment, { key: 0 }, [
                  createTextVNode("[D]")
                ], 64)) : (openBlock(), createElementBlock(Fragment, { key: 1 }, [
                  createTextVNode("[F]")
                ], 64))
              ])
            ]),
            unref(nodeDataInner).editing ? withDirectives((openBlock(), createElementBlock("input", {
              key: 1,
              ref_key: "editingInputRef",
              ref: editingInputRef,
              "onUpdate:modelValue": _cache[1] || (_cache[1] = ($event) => newName.value = $event),
              type: "text",
              onBlur: _cache[2] || (_cache[2] = ($event) => onNodeRename(unref(nodeDataInner), newName.value)),
              onKeyup: withKeys(onEditingEsc, ["esc"]),
              onKeydown: _cache[3] || (_cache[3] = withKeys(($event) => onNodeRename(unref(nodeDataInner), newName.value), ["enter"]))
            }, null, 544)), [
              [vModelText, newName.value]
            ]) : renderSlot(_ctx.$slots, "title", {
              key: 2,
              nodeData: unref(nodeDataInner)
            }, () => [
              createElementVNode("span", _hoisted_2, toDisplayString(unref(nodeDataInner).title), 1)
            ])
          ], 6)
        ], 34), [
          [vShow, unref(nodeDataInner).path !== "/"]
        ]),
        createElementVNode("ul", null, [
          unref(nodeDataInner).addingFolder || unref(nodeDataInner).addingFile || unref(nodeDataInner).expanded || unref(nodeDataInner).path === "/" ? (openBlock(), createElementBlock(Fragment, { key: 0 }, [
            unref(nodeDataInner).type === "folder" && (unref(nodeDataInner).addingFile || unref(nodeDataInner).addingFolder) ? (openBlock(), createElementBlock("li", {
              key: 0,
              style: normalizeStyle({ "padding-left": (props.level < 2 ? 0 : (props.level - 1) * props.levelMargin) + "px" })
            }, [
              unref(nodeDataInner).addingFolder ? withDirectives((openBlock(), createElementBlock("input", {
                key: 0,
                ref_key: "addingFolderInputRef",
                ref: addingFolderInputRef,
                "onUpdate:modelValue": _cache[6] || (_cache[6] = ($event) => newFolderName.value = $event),
                type: "text",
                onBlur: _cache[7] || (_cache[7] = ($event) => onFolderCreate(unref(nodeDataInner), newFolderName.value)),
                onKeydown: _cache[8] || (_cache[8] = withKeys(($event) => onFolderCreate(unref(nodeDataInner), newFolderName.value), ["enter"])),
                onKeyup: withKeys(onFolderEsc, ["esc"])
              }, null, 544)), [
                [vModelText, newFolderName.value]
              ]) : createCommentVNode("", true),
              unref(nodeDataInner).addingFile ? withDirectives((openBlock(), createElementBlock("input", {
                key: 1,
                ref_key: "addingFileInputRef",
                ref: addingFileInputRef,
                "onUpdate:modelValue": _cache[9] || (_cache[9] = ($event) => newFileName.value = $event),
                type: "text",
                onBlur: _cache[10] || (_cache[10] = ($event) => onFileCreate(unref(nodeDataInner), newFileName.value)),
                onKeydown: _cache[11] || (_cache[11] = withKeys(($event) => onFileCreate(unref(nodeDataInner), newFileName.value), ["enter"])),
                onKeyup: withKeys(onFileEsc, ["esc"])
              }, null, 544)), [
                [vModelText, newFileName.value]
              ]) : createCommentVNode("", true)
            ], 4)) : createCommentVNode("", true),
            (openBlock(true), createElementBlock(Fragment, null, renderList(unref(nodeDataInner).children, (item) => {
              return openBlock(), createBlock(_component_FileTreeNode, {
                key: `${item.title}_${item.path}`,
                "node-data": item,
                level: props.level + 1,
                "level-margin": __props.levelMargin,
                draggable: __props.draggable,
                onNodeDrop,
                onNodeSelect,
                onFileCreate,
                onFolderCreate,
                onNodeRename,
                onNodeContextmenu,
                onNodeExpand,
                onNodeCollapse,
                onNodeUpdate
              }, {
                toggler: withCtx(({ nodeData }) => [
                  renderSlot(_ctx.$slots, "toggler", { nodeData })
                ]),
                icon: withCtx(({ nodeData }) => [
                  renderSlot(_ctx.$slots, "icon", { nodeData })
                ]),
                title: withCtx(({ nodeData }) => [
                  renderSlot(_ctx.$slots, "title", { nodeData })
                ]),
                _: 2
              }, 1032, ["node-data", "level", "level-margin", "draggable"]);
            }), 128))
          ], 64)) : createCommentVNode("", true)
        ])
      ], 40, _hoisted_1$1);
    };
  }
});

const FileTreeNode_vue_vue_type_style_index_0_lang = '';

function flattenVisibleTree(tree) {
  const result = [];
  function traverse(node, parentExpanded = false) {
    if (parentExpanded || node.expanded) {
      result.push(node);
      if ((node.path === "/" || node.type === "folder") && node.expanded && node.children) {
        for (const child of node.children) {
          traverse(child, node.expanded);
        }
      }
    }
  }
  traverse(tree);
  return result;
}
function findNodeByPath(node, targetPath, level = 1) {
  if (node.path === targetPath) {
    return node;
  }
  if (node.children && node.children.length > 0) {
    for (const n of node.children) {
      const foundNode = findNodeByPath(n, targetPath, level + 1);
      if (foundNode) {
        return {
          ...foundNode,
          level
        };
      }
    }
  }
  return null;
}
function findParentNodeByPath(parentNode, path) {
  if (!parentNode?.children || parentNode.children.length === 0) {
    return null;
  }
  for (const child of parentNode.children) {
    if (child.path === path) {
      return parentNode;
    }
    const node = findParentNodeByPath(child, path);
    if (node) {
      return node;
    }
  }
  return null;
}
function findIndexByPath(data, path) {
  if (!data) {
    return 0;
  }
  for (let i = 0; i < data.length; i++) {
    if (data[i].path === path) {
      return i;
    }
  }
  return 0;
}
function sep(paths) {
  if (Array.isArray(paths)) {
    for (const path of paths) {
      if (path.includes("/")) {
        return "/";
      }
      if (path.includes("\\")) {
        return "\\";
      }
    }
  } else {
    if (paths.includes("/")) {
      return "/";
    }
    if (paths.includes("\\")) {
      return "\\";
    }
  }
  return /Win\d{2}|Windows/.test(navigator.userAgent) ? "\\" : "/";
}
function join(...paths) {
  const separator = sep(paths);
  const joinedPath = paths.join(separator);
  return joinedPath.replace(/[/\\]{2,}/g, separator);
}
function dirname(path) {
  const separator = sep(path);
  let trimmedPath = path;
  while (trimmedPath.endsWith(separator)) {
    trimmedPath = trimmedPath.slice(0, trimmedPath.length - 1);
  }
  const parts = trimmedPath.split(separator);
  return parts.slice(0, -1).join(separator);
}

const _sfc_main$2 = /* @__PURE__ */ defineComponent({
  __name: "FileTree",
  props: {
    modelValue: {
      type: Array,
      required: true
    },
    draggable: {
      type: Boolean,
      default: false
    },
    levelMargin: {
      type: Number,
      default: 28
    }
  },
  emits: [
    "nodeSelect",
    "fileCreate",
    "folderCreate",
    "nodeRename",
    "nodeContextmenu",
    "nodeExpand",
    "nodeCollapse",
    "nodeDrop",
    "nodeMove",
    "update:modelValue"
  ],
  setup(__props, { emit: __emit }) {
    const props = __props;
    const emits = __emit;
    const ddo = {
      drag: ref(null),
      drop: ref(null),
      position: Position.IN
    };
    provide("ddo", ddo);
    let selectedItems = [];
    let focusedNode = null;
    const treeData = ref({
      title: "/",
      path: "/",
      type: "folder",
      expanded: true,
      children: props.modelValue
    });
    const getSelectedItems = (nodes) => {
      const result = [];
      nodes.forEach((node) => {
        if (node.selected) {
          result.push(node);
        }
        if (node.children) {
          result.push(...getSelectedItems(node.children));
        }
      });
      return result;
    };
    watch(
      () => props.modelValue,
      () => {
        treeData.value = {
          title: "/",
          path: "/",
          type: "folder",
          expanded: true,
          children: props.modelValue
        };
        selectedItems = getSelectedItems(props.modelValue);
      }
    );
    watch(
      treeData,
      (newVal) => {
        emits("update:modelValue", newVal.children || []);
      },
      { deep: true }
    );
    function onFocusIn() {
      window.addEventListener("keydown", onKeyDown);
    }
    function onFocusOut() {
      window.removeEventListener("keydown", onKeyDown);
    }
    function onKeyDown(event) {
      const keyCode = event.code;
      if (keyCode === "ArrowDown" || keyCode === "ArrowUp") {
        event.preventDefault();
        onSelectionMoved(keyCode === "ArrowUp" ? -1 : 1);
      }
    }
    function onNodeExpand(node) {
      emits("nodeExpand", node);
    }
    function onNodeCollapse(node) {
      emits("nodeCollapse", node);
    }
    function onSelectionMoved(direction) {
      if (selectedItems.length === 0) {
        return;
      }
      const currentSelectedItem = selectedItems[0];
      treeData.value.expanded = true;
      const visibleItems = flattenVisibleTree(treeData.value);
      const index = findIndexByPath(visibleItems, currentSelectedItem.path);
      const newIndex = index + direction;
      if (newIndex === 0 || newIndex === visibleItems.length) {
        return;
      }
      selectedItems.forEach((n) => n.selected = false);
      const newSelectedItem = visibleItems[newIndex];
      newSelectedItem.selected = true;
      selectedItems = [newSelectedItem];
    }
    function onNodeContextmenu(event, nodeData) {
      if (focusedNode !== null) {
        focusedNode.focused = false;
      }
      nodeData.focused = true;
      focusedNode = nodeData;
      emits("nodeContextmenu", event, nodeData);
    }
    function onNodeRename(node, title) {
      const oldTitle = node.title;
      const oldPath = node.path;
      const dir = dirname(oldPath);
      const newPath = join(dir, title);
      node.title = title;
      node.path = newPath;
      node.editing = false;
      emits("nodeRename", node, title, oldTitle, newPath, oldPath);
    }
    function onFileCreate(node, title) {
      const newOne = {
        title,
        path: `${node.path}/${title}`,
        type: "file"
      };
      if (!node.children) {
        node.children = [];
      }
      node.children.unshift(newOne);
      emits("fileCreate", node, title);
    }
    function onFolderCreate(node, title) {
      const newOne = {
        title,
        path: `${node.path}/${title}`,
        type: "folder"
      };
      if (!node.children) {
        node.children = [];
      }
      node.children.unshift(newOne);
      emits("folderCreate", node, title);
    }
    function onNodeSelect(event, item) {
      const { path } = item;
      if (focusedNode !== null) {
        focusedNode.focused = false;
        focusedNode = null;
      }
      const isCtrlOrCmdSelect = event.ctrlKey || event.metaKey;
      const isShiftSelect = event.shiftKey;
      if (isCtrlOrCmdSelect && props.draggable) {
        const findIndex = selectedItems.findIndex((item2) => path === item2.path);
        if (findIndex > -1) {
          selectedItems.splice(findIndex, 1);
          item.selected = false;
        } else {
          selectedItems.push(item);
          item.selected = true;
        }
      } else if (isShiftSelect && props.draggable) {
        const visibleItems = flattenVisibleTree(treeData.value);
        let lastIndex;
        const lastSelectedItem = selectedItems.pop();
        if (lastSelectedItem) {
          const lastSelectPath = lastSelectedItem.path;
          lastIndex = visibleItems.findIndex((i) => i.path === lastSelectPath);
        } else {
          lastIndex = 1;
        }
        const currentIndex = visibleItems.findIndex((i) => path === i.path);
        const minIndex = Math.min(lastIndex, currentIndex);
        const maxIndex = Math.max(lastIndex, currentIndex);
        const newSelected = visibleItems.slice(minIndex, maxIndex + 1);
        selectedItems.push(...newSelected);
        newSelected.forEach((s) => s.selected = true);
      } else {
        for (const node of selectedItems) {
          node.selected = false;
        }
        selectedItems.splice(0, selectedItems.length);
        selectedItems.push(item);
        item.selected = true;
      }
      emits("nodeSelect", selectedItems);
    }
    function onNodeDrop() {
      if (ddo.drop.value.path === ddo.drag.value.path) {
        return;
      }
      const dragItem = findNodeByPath(treeData.value, ddo.drag.value.path);
      const dropItem = findNodeByPath(treeData.value, ddo.drop.value.path);
      if (!dragItem || !dropItem) {
        return;
      }
      if (dropItem.path.startsWith(dragItem.path)) {
        return;
      }
      if (dropItem.type === "folder" && dropItem.expanded && ddo.position === Position.BELOW) {
        return;
      }
      if (dragItem.path.startsWith(dropItem.path)) {
        return;
      }
      const dragParent = findParentNodeByPath(treeData.value, dragItem.path);
      if (!dragParent?.children) {
        return;
      }
      for (let i = 0; i < dragParent.children.length; i++) {
        if (dragParent.children[i].title === ddo.drag.value.title) {
          dragParent.children.splice(i, 1);
        }
      }
      const oldPath = dragItem?.path;
      const title = dragItem?.title;
      let newPath;
      if (ddo.position === Position.IN) {
        if (!dropItem.children) {
          dropItem.children = [];
        }
        dropItem.children.push(dragItem);
        newPath = join(dropItem.path, title);
      } else {
        const dropItemParent = findParentNodeByPath(treeData.value, dropItem.path);
        if (!dropItemParent) {
          return;
        }
        if (!dropItemParent.children) {
          dropItemParent.children = [];
        }
        let index = findIndexByPath(dropItemParent.children, dropItem.path);
        if (ddo.position === Position.BELOW) {
          index = index + 1;
        }
        dropItemParent.children.splice(index, 0, dragItem);
        newPath = join(dropItemParent.path, title);
      }
      emits("nodeDrop", newPath, oldPath);
      if (newPath !== oldPath) {
        emits("nodeMove", newPath, oldPath);
      }
    }
    function onNodeUpdate(nodeData) {
      const index = treeData.value.children?.findIndex((item) => item.path === nodeData.path) ?? -1;
      if (index >= 0) {
        treeData.value.children?.splice(index, 1, nodeData);
      }
    }
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        class: "file-tree-view",
        tabindex: "0",
        onFocusin: onFocusIn,
        onFocusout: onFocusOut
      }, [
        createElementVNode("ul", null, [
          createVNode(_sfc_main$3, {
            draggable: __props.draggable,
            level: 1,
            "level-margin": 28,
            "node-data": treeData.value,
            onNodeDrop,
            onNodeSelect,
            onFileCreate,
            onFolderCreate,
            onNodeRename,
            onNodeContextmenu,
            onNodeExpand,
            onNodeCollapse,
            onNodeUpdate
          }, {
            title: withCtx(({ nodeData }) => [
              renderSlot(_ctx.$slots, "title", { nodeData }, void 0, true)
            ]),
            toggler: withCtx(({ nodeData }) => [
              renderSlot(_ctx.$slots, "toggler", { nodeData }, void 0, true)
            ]),
            icon: withCtx(({ nodeData }) => [
              renderSlot(_ctx.$slots, "icon", { nodeData }, void 0, true)
            ]),
            _: 3
          }, 8, ["draggable", "node-data"])
        ])
      ], 32);
    };
  }
});

const FileTree_vue_vue_type_style_index_0_scoped_e9891adc_lang = '';

const _export_sfc = (sfc, props) => {
  const target = sfc.__vccOpts || sfc;
  for (const [key, val] of props) {
    target[key] = val;
  }
  return target;
};

const FileTree = /* @__PURE__ */ _export_sfc(_sfc_main$2, [["__scopeId", "data-v-e9891adc"]]);

function useClickOutside(elementRef, callback) {
  const clickOutsideHandler = (event) => {
    const el = elementRef.value;
    if (!el || el === event.target || event.composedPath().includes(el)) {
      return;
    }
    callback(event);
  };
  onMounted(() => {
    window.addEventListener("click", clickOutsideHandler);
  });
  onBeforeUnmount(() => {
    window.removeEventListener("click", clickOutsideHandler);
  });
}

const _hoisted_1 = { class: "contextMenu-wrapper" };
const _sfc_main$1 = /* @__PURE__ */ defineComponent({
  __name: "ContextMenu",
  emits: ["menuitemClick"],
  setup(__props, { expose: __expose, emit: __emit }) {
    const emits = __emit;
    const contextmenuRef = ref(null);
    const visible = ref(false);
    let contextData = null;
    function onMenuItemClicked(menuId) {
      emits("menuitemClick", contextData, menuId);
      visible.value = false;
    }
    provide("onMenuItemClicked", onMenuItemClicked);
    const position = ref({
      top: 0,
      left: 0
    });
    const style = computed(() => ({
      left: `${position.value.left}px`,
      top: `${position.value.top}px`
    }));
    const hide = () => {
      visible.value = false;
    };
    const calculatePosition = (axis, mousePos, elSize) => {
      const windowSize = axis === "X" ? window.innerWidth : window.innerHeight;
      const scrollPos = axis === "X" ? window.scrollX : window.scrollY;
      let pos = mousePos - scrollPos;
      if (pos + elSize > windowSize) {
        pos = Math.max(0, pos - elSize);
      }
      return pos + scrollPos;
    };
    useClickOutside(contextmenuRef, () => {
      hide();
    });
    const show = async (e, data) => {
      e.preventDefault();
      visible.value = true;
      contextData = data;
      await nextTick();
      const el = contextmenuRef.value;
      if (!el) {
        return;
      }
      const width = el.clientWidth;
      const height = el.clientHeight;
      const { pageX: x, pageY: y } = e;
      position.value.top = calculatePosition("Y", y, height);
      position.value.left = calculatePosition("X", x, width);
    };
    __expose({
      show,
      hide
    });
    return (_ctx, _cache) => {
      return openBlock(), createBlock(Teleport, { to: "body" }, [
        createElementVNode("div", _hoisted_1, [
          createVNode(Transition, { name: "fade" }, {
            default: withCtx(() => [
              visible.value ? (openBlock(), createElementBlock("div", {
                key: 0,
                ref_key: "contextmenuRef",
                ref: contextmenuRef,
                class: "contextMenu",
                style: normalizeStyle(style.value)
              }, [
                renderSlot(_ctx.$slots, "default", {}, void 0, true)
              ], 4)) : createCommentVNode("", true)
            ]),
            _: 3
          })
        ])
      ]);
    };
  }
});

const ContextMenu_vue_vue_type_style_index_0_scoped_87f6d5ad_lang = '';

const ContextMenu = /* @__PURE__ */ _export_sfc(_sfc_main$1, [["__scopeId", "data-v-87f6d5ad"]]);

const _sfc_main = /* @__PURE__ */ defineComponent({
  __name: "ContextMenuItem",
  props: {
    menuId: {
      type: String,
      required: true
    }
  },
  setup(__props) {
    const props = __props;
    const onMenuItemClicked = inject("onMenuItemClicked");
    const handleClick = () => {
      onMenuItemClicked(props.menuId);
    };
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        class: "context-item",
        onClick: handleClick
      }, [
        renderSlot(_ctx.$slots, "default")
      ]);
    };
  }
});

const ContextMenuItem_vue_vue_type_style_index_0_lang = '';

export { ContextMenu, _sfc_main as ContextMenuItem, FileTree, Position };
//# sourceMappingURL=file-tree.es.js.map
