(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global["file-tree"] = {}, global.Vue));
})(this, (function (exports, vue) { 'use strict';

  var Position = /* @__PURE__ */ ((Position2) => {
    Position2["ABOVE"] = "above";
    Position2["IN"] = "in";
    Position2["BELOW"] = "below";
    return Position2;
  })(Position || {});

  const _hoisted_1$1 = ["draggable"];
  const _hoisted_2 = { class: "title" };
  const _sfc_main$3 = /* @__PURE__ */ vue.defineComponent({
    __name: "FileTreeNode",
    props: {
      nodeData: {
        type: Object,
        required: true
      },
      level: {
        type: Number,
        required: true
      },
      draggable: {
        type: Boolean,
        default: false
      },
      levelMargin: {
        type: Number,
        default: 28
      }
    },
    emits: [
      "nodeDrop",
      "nodeSelect",
      "fileCreate",
      "folderCreate",
      "nodeRename",
      "nodeContextmenu",
      "nodeExpand",
      "nodeCollapse",
      "nodeUpdate"
    ],
    setup(__props, { emit: __emit }) {
      const props = __props;
      const emits = __emit;
      let nodeDataInner = vue.ref(props.nodeData);
      vue.watch(() => props.nodeData, () => {
        nodeDataInner.value = props.nodeData;
        newName.value = props.nodeData.title;
      });
      vue.watch(nodeDataInner, () => {
        emits("nodeUpdate", nodeDataInner);
      }, { deep: true });
      const ddo = vue.inject("ddo");
      const editingInputRef = vue.ref();
      const addingFolderInputRef = vue.ref();
      const addingFileInputRef = vue.ref();
      const dragOverClass = vue.ref("");
      const newFileName = vue.ref("");
      const newFolderName = vue.ref("");
      const newName = vue.ref(nodeDataInner.value.title);
      vue.watch(
        () => nodeDataInner.value.editing,
        async (newValue) => {
          if (newValue) {
            newName.value = nodeDataInner.value?.title;
            await vue.nextTick();
            editingInputRef.value.focus();
          }
        }
      );
      vue.watch(
        () => nodeDataInner.value.addingFolder,
        async (newValue) => {
          if (newValue) {
            newFolderName.value = "";
            await vue.nextTick();
            addingFolderInputRef.value.focus();
          }
        }
      );
      vue.watch(
        () => nodeDataInner.value.addingFile,
        async (newValue) => {
          if (newValue) {
            newFileName.value = "";
            await vue.nextTick();
            addingFileInputRef.value.focus();
          }
        }
      );
      const onNodeContextmenu = async (e, nodeData) => {
        emits("nodeContextmenu", e, nodeData);
      };
      function onNodeToggle() {
        nodeDataInner.value.expanded = !nodeDataInner.value.expanded;
        if (nodeDataInner.value.expanded) {
          emits("nodeExpand", nodeDataInner);
        } else {
          emits("nodeCollapse", nodeDataInner);
        }
      }
      function onNodeRename(node, name) {
        if (node.editing) {
          if (name.length > 0) {
            emits("nodeRename", node, name);
          } else {
            onEditingEsc();
          }
        }
      }
      function onEditingEsc() {
        nodeDataInner.value.editing = false;
      }
      function onNodeDrop() {
        emits("nodeDrop");
      }
      function onNodeSelect(e, nodeData) {
        const isCtrlOrCmdSelect = e.ctrlKey || e.metaKey;
        const isShiftSelect = e.shiftKey;
        if (nodeData.type === "folder" && !isCtrlOrCmdSelect && !isShiftSelect && nodeData.path === props.nodeData.path) {
          onNodeToggle();
          if (!props.draggable)
            return;
        }
        emits("nodeSelect", e, nodeData);
      }
      function onFileCreate(nodeData, newFileName2) {
        if (nodeData.addingFile) {
          nodeData.expanded = true;
          if (newFileName2.length > 0) {
            emits("fileCreate", nodeData, newFileName2);
          } else {
            onFileEsc();
          }
        }
      }
      function onFileEsc() {
        nodeDataInner.value.expanded = true;
        nodeDataInner.value.addingFile = false;
      }
      function onFolderCreate(nodeData, newFileName2) {
        if (nodeData.addingFolder) {
          nodeData.expanded = true;
          if (newFileName2.length > 0) {
            emits("folderCreate", nodeData, newFileName2);
          } else {
            onFileEsc();
          }
        }
      }
      function onFolderEsc() {
        nodeDataInner.value.expanded = true;
        nodeDataInner.value.addingFolder = false;
      }
      function onNodeExpand(nodeData) {
        emits("nodeExpand", nodeData);
      }
      function onNodeCollapse(nodeData) {
        emits("nodeCollapse", nodeData);
      }
      function onNodeUpdate(nodeData) {
        const index = nodeDataInner.value.children?.findIndex((item) => item.path === nodeData.path) ?? -1;
        if (index >= 0) {
          nodeDataInner.value.children?.splice(index, 1, nodeData);
        }
      }
      function onDragStart() {
        ddo.drag = nodeDataInner;
      }
      function onDragOver(e) {
        if (e.dataTransfer) {
          e.dataTransfer.dropEffect = "move";
        }
        if (nodeDataInner.value?.path === ddo.drag.path) {
          return;
        }
        const position = calculateDropPosition(e, nodeDataInner.value?.type);
        ddo.position = position;
        if (position === Position.ABOVE) {
          dragOverClass.value = "tree-drag-over-top";
        } else if (position === Position.BELOW) {
          if (nodeDataInner.value.type === "file" || !nodeDataInner.value.expanded) {
            dragOverClass.value = "tree-drag-over-bottom";
          }
        } else {
          dragOverClass.value = "tree-drag-over";
        }
      }
      function calculateDropPosition(e, type) {
        const targetElement = e.target;
        const offsetTop = getOffset(targetElement).top;
        const { offsetHeight } = targetElement;
        const { pageY } = e;
        const gapHeight = type === "folder" ? 0.2 * offsetHeight : 0.5 * offsetHeight;
        if (pageY >= offsetTop + offsetHeight - gapHeight) {
          return Position.BELOW;
        }
        if (pageY < offsetTop + gapHeight) {
          return Position.ABOVE;
        }
        return Position.IN;
      }
      function getOffset(ele) {
        if (!ele.getClientRects().length) {
          return { top: 0, left: 0 };
        }
        const rect = ele.getBoundingClientRect();
        if (rect.width || rect.height) {
          const doc = ele.ownerDocument;
          const win = doc.defaultView;
          const docElem = doc.documentElement;
          if (win) {
            return {
              // 元素距离视窗顶部距离，滚动高度，元素边框厚度
              top: rect.top + win.scrollY - docElem.clientTop,
              left: rect.left + win.scrollX - docElem.clientLeft
            };
          }
        }
        return rect;
      }
      function onDragLeave() {
        dragOverClass.value = "";
      }
      function onDrop() {
        ddo.drop = nodeDataInner;
        emits("nodeDrop");
        dragOverClass.value = "";
        return;
      }
      function onDragEnd() {
        dragOverClass.value = "";
      }
      return (_ctx, _cache) => {
        const _component_FileTreeNode = vue.resolveComponent("FileTreeNode", true);
        return vue.openBlock(), vue.createElementBlock("li", {
          draggable: `${__props.draggable}`,
          class: "file-tree-node",
          onDragstart: vue.withModifiers(onDragStart, ["stop"]),
          onDragend: vue.withModifiers(onDragEnd, ["stop"]),
          onContextmenu: _cache[12] || (_cache[12] = vue.withModifiers(($event) => onNodeContextmenu($event, vue.unref(nodeDataInner)), ["stop", "prevent"]))
        }, [
          vue.withDirectives(vue.createElementVNode("div", {
            class: vue.normalizeClass([
              "node-block",
              {
                selected: vue.unref(nodeDataInner).selected,
                focused: vue.unref(nodeDataInner).focused,
                leaf: !vue.unref(nodeDataInner).children?.length,
                folder: vue.unref(nodeDataInner).type === "folder"
              }
            ]),
            onDragenter: _cache[4] || (_cache[4] = vue.withModifiers(() => {
            }, ["stop"])),
            onDragover: vue.withModifiers(onDragOver, ["stop", "prevent"]),
            onDragleave: vue.withModifiers(onDragLeave, ["stop"]),
            onDrop: vue.withModifiers(onDrop, ["stop"]),
            onClick: _cache[5] || (_cache[5] = vue.withModifiers(($event) => onNodeSelect($event, vue.unref(nodeDataInner)), ["stop"]))
          }, [
            vue.createElementVNode("div", {
              style: vue.normalizeStyle({ "margin-left": (props.level < 3 ? 0 : (props.level - 2) * props.levelMargin) + "px" }),
              class: vue.normalizeClass([dragOverClass.value])
            }, [
              vue.unref(nodeDataInner).type === "folder" ? (vue.openBlock(), vue.createElementBlock("span", {
                key: 0,
                class: "icon",
                onDragover: _cache[0] || (_cache[0] = vue.withModifiers(() => {
                }, ["prevent"]))
              }, [
                vue.renderSlot(_ctx.$slots, "toggler", { nodeData: vue.unref(nodeDataInner) }, () => [
                  vue.unref(nodeDataInner).expanded ? (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 0 }, [
                    vue.createTextVNode("-")
                  ], 64)) : (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 1 }, [
                    vue.createTextVNode("+")
                  ], 64))
                ])
              ], 32)) : vue.createCommentVNode("", true),
              vue.createElementVNode("span", null, [
                vue.renderSlot(_ctx.$slots, "icon", { nodeData: vue.unref(nodeDataInner) }, () => [
                  vue.unref(nodeDataInner).type === "folder" ? (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 0 }, [
                    vue.createTextVNode("[D]")
                  ], 64)) : (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 1 }, [
                    vue.createTextVNode("[F]")
                  ], 64))
                ])
              ]),
              vue.unref(nodeDataInner).editing ? vue.withDirectives((vue.openBlock(), vue.createElementBlock("input", {
                key: 1,
                ref_key: "editingInputRef",
                ref: editingInputRef,
                "onUpdate:modelValue": _cache[1] || (_cache[1] = ($event) => newName.value = $event),
                type: "text",
                onBlur: _cache[2] || (_cache[2] = ($event) => onNodeRename(vue.unref(nodeDataInner), newName.value)),
                onKeyup: vue.withKeys(onEditingEsc, ["esc"]),
                onKeydown: _cache[3] || (_cache[3] = vue.withKeys(($event) => onNodeRename(vue.unref(nodeDataInner), newName.value), ["enter"]))
              }, null, 544)), [
                [vue.vModelText, newName.value]
              ]) : vue.renderSlot(_ctx.$slots, "title", {
                key: 2,
                nodeData: vue.unref(nodeDataInner)
              }, () => [
                vue.createElementVNode("span", _hoisted_2, vue.toDisplayString(vue.unref(nodeDataInner).title), 1)
              ])
            ], 6)
          ], 34), [
            [vue.vShow, vue.unref(nodeDataInner).path !== "/"]
          ]),
          vue.createElementVNode("ul", null, [
            vue.unref(nodeDataInner).addingFolder || vue.unref(nodeDataInner).addingFile || vue.unref(nodeDataInner).expanded || vue.unref(nodeDataInner).path === "/" ? (vue.openBlock(), vue.createElementBlock(vue.Fragment, { key: 0 }, [
              vue.unref(nodeDataInner).type === "folder" && (vue.unref(nodeDataInner).addingFile || vue.unref(nodeDataInner).addingFolder) ? (vue.openBlock(), vue.createElementBlock("li", {
                key: 0,
                style: vue.normalizeStyle({ "padding-left": (props.level < 2 ? 0 : (props.level - 1) * props.levelMargin) + "px" })
              }, [
                vue.unref(nodeDataInner).addingFolder ? vue.withDirectives((vue.openBlock(), vue.createElementBlock("input", {
                  key: 0,
                  ref_key: "addingFolderInputRef",
                  ref: addingFolderInputRef,
                  "onUpdate:modelValue": _cache[6] || (_cache[6] = ($event) => newFolderName.value = $event),
                  type: "text",
                  onBlur: _cache[7] || (_cache[7] = ($event) => onFolderCreate(vue.unref(nodeDataInner), newFolderName.value)),
                  onKeydown: _cache[8] || (_cache[8] = vue.withKeys(($event) => onFolderCreate(vue.unref(nodeDataInner), newFolderName.value), ["enter"])),
                  onKeyup: vue.withKeys(onFolderEsc, ["esc"])
                }, null, 544)), [
                  [vue.vModelText, newFolderName.value]
                ]) : vue.createCommentVNode("", true),
                vue.unref(nodeDataInner).addingFile ? vue.withDirectives((vue.openBlock(), vue.createElementBlock("input", {
                  key: 1,
                  ref_key: "addingFileInputRef",
                  ref: addingFileInputRef,
                  "onUpdate:modelValue": _cache[9] || (_cache[9] = ($event) => newFileName.value = $event),
                  type: "text",
                  onBlur: _cache[10] || (_cache[10] = ($event) => onFileCreate(vue.unref(nodeDataInner), newFileName.value)),
                  onKeydown: _cache[11] || (_cache[11] = vue.withKeys(($event) => onFileCreate(vue.unref(nodeDataInner), newFileName.value), ["enter"])),
                  onKeyup: vue.withKeys(onFileEsc, ["esc"])
                }, null, 544)), [
                  [vue.vModelText, newFileName.value]
                ]) : vue.createCommentVNode("", true)
              ], 4)) : vue.createCommentVNode("", true),
              (vue.openBlock(true), vue.createElementBlock(vue.Fragment, null, vue.renderList(vue.unref(nodeDataInner).children, (item) => {
                return vue.openBlock(), vue.createBlock(_component_FileTreeNode, {
                  key: `${item.title}_${item.path}`,
                  "node-data": item,
                  level: props.level + 1,
                  "level-margin": __props.levelMargin,
                  draggable: __props.draggable,
                  onNodeDrop,
                  onNodeSelect,
                  onFileCreate,
                  onFolderCreate,
                  onNodeRename,
                  onNodeContextmenu,
                  onNodeExpand,
                  onNodeCollapse,
                  onNodeUpdate
                }, {
                  toggler: vue.withCtx(({ nodeData }) => [
                    vue.renderSlot(_ctx.$slots, "toggler", { nodeData })
                  ]),
                  icon: vue.withCtx(({ nodeData }) => [
                    vue.renderSlot(_ctx.$slots, "icon", { nodeData })
                  ]),
                  title: vue.withCtx(({ nodeData }) => [
                    vue.renderSlot(_ctx.$slots, "title", { nodeData })
                  ]),
                  _: 2
                }, 1032, ["node-data", "level", "level-margin", "draggable"]);
              }), 128))
            ], 64)) : vue.createCommentVNode("", true)
          ])
        ], 40, _hoisted_1$1);
      };
    }
  });

  const FileTreeNode_vue_vue_type_style_index_0_lang = '';

  function flattenVisibleTree(tree) {
    const result = [];
    function traverse(node, parentExpanded = false) {
      if (parentExpanded || node.expanded) {
        result.push(node);
        if ((node.path === "/" || node.type === "folder") && node.expanded && node.children) {
          for (const child of node.children) {
            traverse(child, node.expanded);
          }
        }
      }
    }
    traverse(tree);
    return result;
  }
  function findNodeByPath(node, targetPath, level = 1) {
    if (node.path === targetPath) {
      return node;
    }
    if (node.children && node.children.length > 0) {
      for (const n of node.children) {
        const foundNode = findNodeByPath(n, targetPath, level + 1);
        if (foundNode) {
          return {
            ...foundNode,
            level
          };
        }
      }
    }
    return null;
  }
  function findParentNodeByPath(parentNode, path) {
    if (!parentNode?.children || parentNode.children.length === 0) {
      return null;
    }
    for (const child of parentNode.children) {
      if (child.path === path) {
        return parentNode;
      }
      const node = findParentNodeByPath(child, path);
      if (node) {
        return node;
      }
    }
    return null;
  }
  function findIndexByPath(data, path) {
    if (!data) {
      return 0;
    }
    for (let i = 0; i < data.length; i++) {
      if (data[i].path === path) {
        return i;
      }
    }
    return 0;
  }
  function sep(paths) {
    if (Array.isArray(paths)) {
      for (const path of paths) {
        if (path.includes("/")) {
          return "/";
        }
        if (path.includes("\\")) {
          return "\\";
        }
      }
    } else {
      if (paths.includes("/")) {
        return "/";
      }
      if (paths.includes("\\")) {
        return "\\";
      }
    }
    return /Win\d{2}|Windows/.test(navigator.userAgent) ? "\\" : "/";
  }
  function join(...paths) {
    const separator = sep(paths);
    const joinedPath = paths.join(separator);
    return joinedPath.replace(/[/\\]{2,}/g, separator);
  }
  function dirname(path) {
    const separator = sep(path);
    let trimmedPath = path;
    while (trimmedPath.endsWith(separator)) {
      trimmedPath = trimmedPath.slice(0, trimmedPath.length - 1);
    }
    const parts = trimmedPath.split(separator);
    return parts.slice(0, -1).join(separator);
  }

  const _sfc_main$2 = /* @__PURE__ */ vue.defineComponent({
    __name: "FileTree",
    props: {
      modelValue: {
        type: Array,
        required: true
      },
      draggable: {
        type: Boolean,
        default: false
      },
      levelMargin: {
        type: Number,
        default: 28
      }
    },
    emits: [
      "nodeSelect",
      "fileCreate",
      "folderCreate",
      "nodeRename",
      "nodeContextmenu",
      "nodeExpand",
      "nodeCollapse",
      "nodeDrop",
      "nodeMove",
      "update:modelValue"
    ],
    setup(__props, { emit: __emit }) {
      const props = __props;
      const emits = __emit;
      const ddo = {
        drag: vue.ref(null),
        drop: vue.ref(null),
        position: Position.IN
      };
      vue.provide("ddo", ddo);
      let selectedItems = [];
      let focusedNode = null;
      const treeData = vue.ref({
        title: "/",
        path: "/",
        type: "folder",
        expanded: true,
        children: props.modelValue
      });
      const getSelectedItems = (nodes) => {
        const result = [];
        nodes.forEach((node) => {
          if (node.selected) {
            result.push(node);
          }
          if (node.children) {
            result.push(...getSelectedItems(node.children));
          }
        });
        return result;
      };
      vue.watch(
        () => props.modelValue,
        () => {
          treeData.value = {
            title: "/",
            path: "/",
            type: "folder",
            expanded: true,
            children: props.modelValue
          };
          selectedItems = getSelectedItems(props.modelValue);
        }
      );
      vue.watch(
        treeData,
        (newVal) => {
          emits("update:modelValue", newVal.children || []);
        },
        { deep: true }
      );
      function onFocusIn() {
        window.addEventListener("keydown", onKeyDown);
      }
      function onFocusOut() {
        window.removeEventListener("keydown", onKeyDown);
      }
      function onKeyDown(event) {
        const keyCode = event.code;
        if (keyCode === "ArrowDown" || keyCode === "ArrowUp") {
          event.preventDefault();
          onSelectionMoved(keyCode === "ArrowUp" ? -1 : 1);
        }
      }
      function onNodeExpand(node) {
        emits("nodeExpand", node);
      }
      function onNodeCollapse(node) {
        emits("nodeCollapse", node);
      }
      function onSelectionMoved(direction) {
        if (selectedItems.length === 0) {
          return;
        }
        const currentSelectedItem = selectedItems[0];
        treeData.value.expanded = true;
        const visibleItems = flattenVisibleTree(treeData.value);
        const index = findIndexByPath(visibleItems, currentSelectedItem.path);
        const newIndex = index + direction;
        if (newIndex === 0 || newIndex === visibleItems.length) {
          return;
        }
        selectedItems.forEach((n) => n.selected = false);
        const newSelectedItem = visibleItems[newIndex];
        newSelectedItem.selected = true;
        selectedItems = [newSelectedItem];
      }
      function onNodeContextmenu(event, nodeData) {
        if (focusedNode !== null) {
          focusedNode.focused = false;
        }
        nodeData.focused = true;
        focusedNode = nodeData;
        emits("nodeContextmenu", event, nodeData);
      }
      function onNodeRename(node, title) {
        const oldTitle = node.title;
        const oldPath = node.path;
        const dir = dirname(oldPath);
        const newPath = join(dir, title);
        node.title = title;
        node.path = newPath;
        node.editing = false;
        emits("nodeRename", node, title, oldTitle, newPath, oldPath);
      }
      function onFileCreate(node, title) {
        const newOne = {
          title,
          path: `${node.path}/${title}`,
          type: "file"
        };
        if (!node.children) {
          node.children = [];
        }
        node.children.unshift(newOne);
        emits("fileCreate", node, title);
      }
      function onFolderCreate(node, title) {
        const newOne = {
          title,
          path: `${node.path}/${title}`,
          type: "folder"
        };
        if (!node.children) {
          node.children = [];
        }
        node.children.unshift(newOne);
        emits("folderCreate", node, title);
      }
      function onNodeSelect(event, item) {
        const { path } = item;
        if (focusedNode !== null) {
          focusedNode.focused = false;
          focusedNode = null;
        }
        const isCtrlOrCmdSelect = event.ctrlKey || event.metaKey;
        const isShiftSelect = event.shiftKey;
        if (isCtrlOrCmdSelect && props.draggable) {
          const findIndex = selectedItems.findIndex((item2) => path === item2.path);
          if (findIndex > -1) {
            selectedItems.splice(findIndex, 1);
            item.selected = false;
          } else {
            selectedItems.push(item);
            item.selected = true;
          }
        } else if (isShiftSelect && props.draggable) {
          const visibleItems = flattenVisibleTree(treeData.value);
          let lastIndex;
          const lastSelectedItem = selectedItems.pop();
          if (lastSelectedItem) {
            const lastSelectPath = lastSelectedItem.path;
            lastIndex = visibleItems.findIndex((i) => i.path === lastSelectPath);
          } else {
            lastIndex = 1;
          }
          const currentIndex = visibleItems.findIndex((i) => path === i.path);
          const minIndex = Math.min(lastIndex, currentIndex);
          const maxIndex = Math.max(lastIndex, currentIndex);
          const newSelected = visibleItems.slice(minIndex, maxIndex + 1);
          selectedItems.push(...newSelected);
          newSelected.forEach((s) => s.selected = true);
        } else {
          for (const node of selectedItems) {
            node.selected = false;
          }
          selectedItems.splice(0, selectedItems.length);
          selectedItems.push(item);
          item.selected = true;
        }
        emits("nodeSelect", selectedItems);
      }
      function onNodeDrop() {
        if (ddo.drop.value.path === ddo.drag.value.path) {
          return;
        }
        const dragItem = findNodeByPath(treeData.value, ddo.drag.value.path);
        const dropItem = findNodeByPath(treeData.value, ddo.drop.value.path);
        if (!dragItem || !dropItem) {
          return;
        }
        if (dropItem.path.startsWith(dragItem.path)) {
          return;
        }
        if (dropItem.type === "folder" && dropItem.expanded && ddo.position === Position.BELOW) {
          return;
        }
        if (dragItem.path.startsWith(dropItem.path)) {
          return;
        }
        const dragParent = findParentNodeByPath(treeData.value, dragItem.path);
        if (!dragParent?.children) {
          return;
        }
        for (let i = 0; i < dragParent.children.length; i++) {
          if (dragParent.children[i].title === ddo.drag.value.title) {
            dragParent.children.splice(i, 1);
          }
        }
        const oldPath = dragItem?.path;
        const title = dragItem?.title;
        let newPath;
        if (ddo.position === Position.IN) {
          if (!dropItem.children) {
            dropItem.children = [];
          }
          dropItem.children.push(dragItem);
          newPath = join(dropItem.path, title);
        } else {
          const dropItemParent = findParentNodeByPath(treeData.value, dropItem.path);
          if (!dropItemParent) {
            return;
          }
          if (!dropItemParent.children) {
            dropItemParent.children = [];
          }
          let index = findIndexByPath(dropItemParent.children, dropItem.path);
          if (ddo.position === Position.BELOW) {
            index = index + 1;
          }
          dropItemParent.children.splice(index, 0, dragItem);
          newPath = join(dropItemParent.path, title);
        }
        emits("nodeDrop", newPath, oldPath);
        if (newPath !== oldPath) {
          emits("nodeMove", newPath, oldPath);
        }
      }
      function onNodeUpdate(nodeData) {
        const index = treeData.value.children?.findIndex((item) => item.path === nodeData.path) ?? -1;
        if (index >= 0) {
          treeData.value.children?.splice(index, 1, nodeData);
        }
      }
      return (_ctx, _cache) => {
        return vue.openBlock(), vue.createElementBlock("div", {
          class: "file-tree-view",
          tabindex: "0",
          onFocusin: onFocusIn,
          onFocusout: onFocusOut
        }, [
          vue.createElementVNode("ul", null, [
            vue.createVNode(_sfc_main$3, {
              draggable: __props.draggable,
              level: 1,
              "level-margin": 28,
              "node-data": treeData.value,
              onNodeDrop,
              onNodeSelect,
              onFileCreate,
              onFolderCreate,
              onNodeRename,
              onNodeContextmenu,
              onNodeExpand,
              onNodeCollapse,
              onNodeUpdate
            }, {
              title: vue.withCtx(({ nodeData }) => [
                vue.renderSlot(_ctx.$slots, "title", { nodeData }, void 0, true)
              ]),
              toggler: vue.withCtx(({ nodeData }) => [
                vue.renderSlot(_ctx.$slots, "toggler", { nodeData }, void 0, true)
              ]),
              icon: vue.withCtx(({ nodeData }) => [
                vue.renderSlot(_ctx.$slots, "icon", { nodeData }, void 0, true)
              ]),
              _: 3
            }, 8, ["draggable", "node-data"])
          ])
        ], 32);
      };
    }
  });

  const FileTree_vue_vue_type_style_index_0_scoped_e9891adc_lang = '';

  const _export_sfc = (sfc, props) => {
    const target = sfc.__vccOpts || sfc;
    for (const [key, val] of props) {
      target[key] = val;
    }
    return target;
  };

  const FileTree = /* @__PURE__ */ _export_sfc(_sfc_main$2, [["__scopeId", "data-v-e9891adc"]]);

  function useClickOutside(elementRef, callback) {
    const clickOutsideHandler = (event) => {
      const el = elementRef.value;
      if (!el || el === event.target || event.composedPath().includes(el)) {
        return;
      }
      callback(event);
    };
    vue.onMounted(() => {
      window.addEventListener("click", clickOutsideHandler);
    });
    vue.onBeforeUnmount(() => {
      window.removeEventListener("click", clickOutsideHandler);
    });
  }

  const _hoisted_1 = { class: "contextMenu-wrapper" };
  const _sfc_main$1 = /* @__PURE__ */ vue.defineComponent({
    __name: "ContextMenu",
    emits: ["menuitemClick"],
    setup(__props, { expose: __expose, emit: __emit }) {
      const emits = __emit;
      const contextmenuRef = vue.ref(null);
      const visible = vue.ref(false);
      let contextData = null;
      function onMenuItemClicked(menuId) {
        emits("menuitemClick", contextData, menuId);
        visible.value = false;
      }
      vue.provide("onMenuItemClicked", onMenuItemClicked);
      const position = vue.ref({
        top: 0,
        left: 0
      });
      const style = vue.computed(() => ({
        left: `${position.value.left}px`,
        top: `${position.value.top}px`
      }));
      const hide = () => {
        visible.value = false;
      };
      const calculatePosition = (axis, mousePos, elSize) => {
        const windowSize = axis === "X" ? window.innerWidth : window.innerHeight;
        const scrollPos = axis === "X" ? window.scrollX : window.scrollY;
        let pos = mousePos - scrollPos;
        if (pos + elSize > windowSize) {
          pos = Math.max(0, pos - elSize);
        }
        return pos + scrollPos;
      };
      useClickOutside(contextmenuRef, () => {
        hide();
      });
      const show = async (e, data) => {
        e.preventDefault();
        visible.value = true;
        contextData = data;
        await vue.nextTick();
        const el = contextmenuRef.value;
        if (!el) {
          return;
        }
        const width = el.clientWidth;
        const height = el.clientHeight;
        const { pageX: x, pageY: y } = e;
        position.value.top = calculatePosition("Y", y, height);
        position.value.left = calculatePosition("X", x, width);
      };
      __expose({
        show,
        hide
      });
      return (_ctx, _cache) => {
        return vue.openBlock(), vue.createBlock(vue.Teleport, { to: "body" }, [
          vue.createElementVNode("div", _hoisted_1, [
            vue.createVNode(vue.Transition, { name: "fade" }, {
              default: vue.withCtx(() => [
                visible.value ? (vue.openBlock(), vue.createElementBlock("div", {
                  key: 0,
                  ref_key: "contextmenuRef",
                  ref: contextmenuRef,
                  class: "contextMenu",
                  style: vue.normalizeStyle(style.value)
                }, [
                  vue.renderSlot(_ctx.$slots, "default", {}, void 0, true)
                ], 4)) : vue.createCommentVNode("", true)
              ]),
              _: 3
            })
          ])
        ]);
      };
    }
  });

  const ContextMenu_vue_vue_type_style_index_0_scoped_87f6d5ad_lang = '';

  const ContextMenu = /* @__PURE__ */ _export_sfc(_sfc_main$1, [["__scopeId", "data-v-87f6d5ad"]]);

  const _sfc_main = /* @__PURE__ */ vue.defineComponent({
    __name: "ContextMenuItem",
    props: {
      menuId: {
        type: String,
        required: true
      }
    },
    setup(__props) {
      const props = __props;
      const onMenuItemClicked = vue.inject("onMenuItemClicked");
      const handleClick = () => {
        onMenuItemClicked(props.menuId);
      };
      return (_ctx, _cache) => {
        return vue.openBlock(), vue.createElementBlock("div", {
          class: "context-item",
          onClick: handleClick
        }, [
          vue.renderSlot(_ctx.$slots, "default")
        ]);
      };
    }
  });

  const ContextMenuItem_vue_vue_type_style_index_0_lang = '';

  exports.ContextMenu = ContextMenu;
  exports.ContextMenuItem = _sfc_main;
  exports.FileTree = FileTree;
  exports.Position = Position;

  Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });

}));
//# sourceMappingURL=file-tree.umd.js.map
